using ComputersShopBusinessLogic.BusinessLogics;
using ComputersShopBusinessLogic.OfficePackage.Implements;
using ComputersShopBusinessLogic.OfficePackage;
using ComputersShopContracts.BusinessLogicContracts;
using ComputersShopContracts.StoragesContracts;
using ComputersShopDataBaseImplement.Implements;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using ComputersShopBusinessLogic.MailWorker;
using ComputersShopContracts.BindingModels;
using ComputersShopContracts.DI;

namespace ComputersShopView
{
    internal static class Program
	{
		/// <summary>
		///  The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main()
		{
			// To customize application configuration such as set high DPI settings or default font,
			// see https://aka.ms/applicationconfiguration.
			ApplicationConfiguration.Initialize();
			InitDependency();
			try
            {
                var mailSender = DependencyManager.Instance.Resolve<AbstractMailWorker>();
                mailSender?.MailConfig(new MailConfigBindingModel
                {
                    MailLogin = System.Configuration.ConfigurationManager.AppSettings["MailLogin"] ?? string.Empty,
                    MailPassword = System.Configuration.ConfigurationManager.AppSettings["MailPassword"] ?? string.Empty,
                    SmtpClientHost = System.Configuration.ConfigurationManager.AppSettings["SmtpClientHost"] ?? string.Empty,
                    SmtpClientPort = Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["SmtpClientPort"]),
                    PopHost = System.Configuration.ConfigurationManager.AppSettings["PopHost"] ?? string.Empty,
                    PopPort = Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["PopPort"])
                });

                var timer = new System.Threading.Timer(new TimerCallback(MailCheck!), null, 0, 100000);
            }
            catch (Exception ex)
            {
				var logger = DependencyManager.Instance.Resolve<ILogger>();
				logger?.LogError(ex, "Error");
            }
            Application.Run(DependencyManager.Instance.Resolve<FormMain>());
		}

		/*private static void ConfigureServices(ServiceCollection services)
		{
			services.AddLogging(option =>
			{
				option.SetMinimumLevel(LogLevel.Information);
				option.AddNLog("nlog.config");
			});
			services.AddTransient<IComponentStorage, ComponentStorage>();
			services.AddTransient<IComputerStorage, ComputerStorage>();
			services.AddTransient<IOrderStorage, OrderStorage>();
			services.AddTransient<IClientStorage, ClientStorage>();
			services.AddTransient<IImplementerStorage, ImplementerStorage>();
			services.AddTransient<IMessageInfoStorage, MessageInfoStorage>();
			services.AddTransient<IBackUpInfo, BackUpInfo>();

			services.AddTransient<IComponentLogic, ComponentLogic>();
			services.AddTransient<IComputerLogic, ComputerLogic>();
			services.AddTransient<IOrderLogic, OrderLogic>();
			services.AddTransient<IReportLogic, ReportLogic>();
			services.AddTransient<IClientLogic, ClientLogic>();
			services.AddTransient<IImplementerLogic, ImplementerLogic>();
			services.AddTransient<IWorkProcess, WorkModeling>();
			services.AddTransient<IMessageInfoLogic, MessageInfoLogic>();
			services.AddTransient<IBackUpLogic, BackUpLogic>();

			services.AddTransient<AbstractSaveToWord, SaveToWord>();
			services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
			services.AddTransient<AbstractSaveToPdf, SaveToPdf>();
			services.AddSingleton<AbstractMailWorker, MailKitWorker>();

			services.AddTransient<FormMain>();
			services.AddTransient<FormClients>();
			services.AddTransient<FormComponent>();
			services.AddTransient<FormComponents>();
			services.AddTransient<FormCreateOrder>();
			services.AddTransient<FormComputer>();
			services.AddTransient<FormComputers>();
			services.AddTransient<FormComputerComponent>();
			services.AddTransient<FormReportComputerComponents>();
			services.AddTransient<FormReportOrders>();
			services.AddTransient<FormImplementer>();
			services.AddTransient<FormImplementers>();
			services.AddTransient<FormMails>();
		}*/

		private static void InitDependency()
		{
			DependencyManager.InitDependency();
			DependencyManager.Instance.AddLogging(option =>
			{
				option.SetMinimumLevel(LogLevel.Information);
				option.AddNLog("nlog.config");
			});
			DependencyManager.Instance.RegisterType<IComponentLogic, ComponentLogic>();
			DependencyManager.Instance.RegisterType<IOrderLogic, OrderLogic>();
			DependencyManager.Instance.RegisterType<IComputerLogic, ComputerLogic>();
			DependencyManager.Instance.RegisterType<IReportLogic, ReportLogic>();
			DependencyManager.Instance.RegisterType<IClientLogic, ClientLogic>();
			DependencyManager.Instance.RegisterType<IImplementerLogic, ImplementerLogic>();
			DependencyManager.Instance.RegisterType<IMessageInfoLogic, MessageInfoLogic>();
			DependencyManager.Instance.RegisterType<AbstractSaveToExcel, SaveToExcel>();
			DependencyManager.Instance.RegisterType<AbstractSaveToWord, SaveToWord>();
			DependencyManager.Instance.RegisterType<AbstractSaveToPdf, SaveToPdf>();
			DependencyManager.Instance.RegisterType<IWorkProcess, WorkModeling>();
			DependencyManager.Instance.RegisterType<AbstractMailWorker, MailKitWorker>(true);
			DependencyManager.Instance.RegisterType<IBackUpLogic, BackUpLogic>();
			DependencyManager.Instance.RegisterType<FormMain>();
			DependencyManager.Instance.RegisterType<FormComponent>();
			DependencyManager.Instance.RegisterType<FormComponents>();
			DependencyManager.Instance.RegisterType<FormComputers>();
			DependencyManager.Instance.RegisterType<FormCreateOrder>();
			DependencyManager.Instance.RegisterType<FormComputer>();
			DependencyManager.Instance.RegisterType<FormComputerComponent>();
			DependencyManager.Instance.RegisterType<FormReportComputerComponents>();
			DependencyManager.Instance.RegisterType<FormReportOrders>();
			DependencyManager.Instance.RegisterType<FormClients>();
			DependencyManager.Instance.RegisterType<FormImplementer>();
			DependencyManager.Instance.RegisterType<FormImplementers>();
			DependencyManager.Instance.RegisterType<FormMails>();
		}
		private static void MailCheck(object obj) => DependencyManager.Instance.Resolve<AbstractMailWorker>()?.MailCheck();
	}
}